@@ -1,39 +1,8 @@
 """Utility methods for CLI."""
-from typing import List
-
 import click
 
-from statue.command import Command
 from statue.verbosity import DEFAULT_VERBOSITY, SILENT, VERBOSE, VERBOSITIES
 
-
-def install_commands_if_missing(
-    commands_list: List[Command], verbosity: str = DEFAULT_VERBOSITY
-) -> None:
-    """
-    Install commands if missing using `pip install`.
-
-    :param commands_list: list of :class:`Command` items
-    :param verbosity: String. Verbosity level
-    """
-    uninstalled_commands = [
-        command for command in commands_list if not command.installed()
-    ]
-    if len(uninstalled_commands) == 0:
-        print("All commands are installed!")
-    else:
-        installed = []
-        print(
-            "The following commands are not installed: "
-            f"{', '.join([command.name for command in uninstalled_commands])}"
-        )
-        for command in uninstalled_commands:
-            if command.name in installed:
-                continue
-            command.install(verbosity)
-            installed.append(command.name)
-
-
 contexts_option = click.option(
     "-c",
     "--context",
@@ -62,3 +31,47 @@ silent_option = click.option(
 verbose_option = click.option(
     "--verbose", "verbosity", flag_value=VERBOSE, help=f'Set verbosity to "{VERBOSE}".'
 )
+
+
+def name_style(name):
+    """
+    Styling function to emphasise names.
+
+    :param name: The name to style
+    :return: Styled name
+    :rtype: str
+    """
+    return click.style(name, fg="magenta")
+
+
+def bullet_style(bullet_name):
+    """
+    Styling function to emphasise bullet names.
+
+    :param bullet_name: The name to style
+    :return: Styled bullet name
+    :rtype: str
+    """
+    return click.style(bullet_name, fg="yellow")
+
+
+def success_style(success_string):
+    """
+    Styling function to emphasise bullet names.
+
+    :param success_string: The string to style
+    :return: Styled success string
+    :rtype: str
+    """
+    return click.style(success_string, fg="green")
+
+
+def failure_style(failure_string):
+    """
+    Styling function to emphasise bullet names.
+
+    :param failure_string: The string to style
+    :return: Styled failure string
+    :rtype: str
+    """
+    return click.style(failure_string, fg="red")
