@@ -1,7 +1,7 @@
 """Get Statue global configuration."""
 from copy import deepcopy
 from pathlib import Path
-from typing import Any, List, MutableMapping, Optional, Union
+from typing import Any, Dict, List, MutableMapping, Optional, Union
 
 import toml
 
@@ -15,15 +15,15 @@ from statue.constants import (
     DEFAULT_CONFIGURATION_FILE,
     HELP,
     OVERRIDE,
-    PARENT,
     SOURCES,
     STANDARD,
     STATUE,
+    VERSION,
 )
+from statue.context import Context
 from statue.exceptions import (
     EmptyConfiguration,
     InvalidCommand,
-    InvalidStatueConfiguration,
     MissingConfiguration,
     UnknownCommand,
     UnknownContext,
@@ -38,14 +38,26 @@ class Configuration:
 
     @classmethod
     def configuration_path(cls, directory: Union[Path, str]) -> Path:
-        """Get default path of configuration file in directory."""
+        """
+        Get default path of configuration file in directory.
+
+        :param directory: Directory in which the configuration path is supposed to be
+        :type directory: Path or str
+        :return: Configuration path location
+        :rtype: Path
+        """
         if isinstance(directory, str):
             directory = Path(directory)
         return directory / "statue.toml"
 
     @classmethod
     def default_configuration(cls) -> Optional[MutableMapping[str, Any]]:
-        """Getter of default configuration."""
+        """
+        Default configuration path.
+
+        :return: Default configuration path location
+        :rtype: Path
+        """
         if cls.__default_configuration is None:
             cls.__load_default_configuration()
         return deepcopy(cls.__default_configuration)
@@ -54,12 +66,25 @@ class Configuration:
     def set_default_configuration(
         cls, default_configuration: Optional[MutableMapping[str, Any]]
     ) -> None:
-        """Setter of default configuration."""
+        """
+        Setter of default configuration.
+
+        :param default_configuration: Configuration to be saved
+        :type default_configuration: None or mutable mapping
+        """
         cls.__default_configuration = default_configuration
 
     @classmethod
     def statue_configuration(cls) -> MutableMapping[str, Any]:
-        """Getter of general statue configuration."""
+        """
+        Getter of general statue configuration.
+
+        If not set, take default configuration instead.
+
+        :return: Configuration dictionary of Statue
+        :rtype: MutableMapping[str, Any]
+        :raises EmptyConfiguration: Raised if no configuration was set.
+        """
         if cls.__statue_configuration is not None:
             return deepcopy(cls.__statue_configuration)
         default_configuration = cls.default_configuration()
@@ -71,17 +96,32 @@ class Configuration:
     def set_statue_configuration(
         cls, statue_configuration: Optional[MutableMapping[str, Any]]
     ) -> None:
-        """Setter of general statue configuration."""
+        """
+        Setter of general statue configuration.
+
+        :param statue_configuration: Configuration to be saved
+        :type statue_configuration: None or mutable mapping
+        """
         cls.__statue_configuration = statue_configuration
 
     @classmethod
     def commands_configuration(cls) -> Optional[MutableMapping[str, Any]]:
-        """Getter of the commands configuration."""
+        """
+        Getter of the commands configuration.
+
+        :return: Commands configuration dictionary
+        :rtype: MutableMapping[str, Any]
+        """
         return cls.statue_configuration().get(COMMANDS, None)
 
     @classmethod
     def commands_names_list(cls) -> List[str]:
-        """Getter of the commands list."""
+        """
+        List of all available commands.
+
+        :return: Available commands list
+        :rtype: List[str]
+        """
         commands_configuration = cls.commands_configuration()
         if commands_configuration is None:
             return []
@@ -97,8 +137,8 @@ class Configuration:
         :param command_name: Name of the desired command.
         :type command_name: str
         :return: configuration dictionary.
-        :raises: raise :Class:`MissingConfiguration` if no contexts configuration was
-        set.
+        :rtype: None or mutable mapping
+        :raises MissingConfiguration: raised when no contexts configuration was set.
         """
         commands_configuration = cls.commands_configuration()
         if commands_configuration is None:
@@ -108,60 +148,99 @@ class Configuration:
     @classmethod
     def sources_configuration(
         cls,
-    ) -> MutableMapping[str, MutableMapping[str, Any]]:
-        """Getter of the sources configuration."""
+    ) -> MutableMapping[Path, MutableMapping[str, Any]]:
+        """
+        Getter of the sources configuration.
+
+        :return: Sources configuration dictionary
+        :rtype: MutableMapping[str, Any]
+        :raises MissingConfiguration: raised if no sources configuration was set.
+        """
         sources_configuration: Optional[
-            MutableMapping[str, MutableMapping[str, Any]]
+            MutableMapping[Path, MutableMapping[str, Any]]
         ] = cls.statue_configuration().get(SOURCES, None)
         if sources_configuration is None:
             raise MissingConfiguration(SOURCES)
         return sources_configuration
 
     @classmethod
-    def sources_list(cls) -> List[str]:
-        """Getter of the commands list."""
+    def sources_list(cls) -> List[Path]:
+        """
+        List of sources to run statue over, as specified in the configuration file.
+
+        :return: Available sources list
+        :rtype: List[Path]
+        """
         return list(cls.sources_configuration().keys())
 
     @classmethod
     def get_source_configuration(
         cls, source: Union[Path, str]
-    ) -> MutableMapping[str, Any]:
+    ) -> Optional[MutableMapping[str, Any]]:
         """
         Get configuration dictionary of a context.
 
         :param source: Name of the desired source.
         :type source: str
         :return: configuration dictionary.
-        :raises: raise :Class:`MissingConfiguration` if no contexts configuration was
-        set.
+        :rtype: None or mutable mapping
         """
-        source_configuration = cls.sources_configuration()
-        if isinstance(source, Path):
-            source = str(source)
-        return source_configuration.get(source, {})
+        sources_configuration = cls.sources_configuration()
+        if not isinstance(source, Path):
+            source = Path(source)
+        for source_path, setup in sources_configuration.items():
+            try:
+                source.relative_to(source_path)
+                return setup
+            except ValueError:
+                continue
+        return None
 
     @classmethod
-    def contexts_configuration(cls) -> Optional[MutableMapping[str, Any]]:
-        """Getter of the contexts configuration."""
+    def contexts_map(cls) -> Optional[Dict[str, Context]]:
+        """
+        Contexts configuration.
+
+        :return: Sources configuration dictionary
+        :rtype: None or dictionary
+        """
         return cls.statue_configuration().get(CONTEXTS, None)
 
     @classmethod
-    def get_context_configuration(
-        cls, context_name: str
-    ) -> Optional[MutableMapping[str, Any]]:
+    def contexts_list(cls) -> List[Context]:
+        """
+        List of all available contexts.
+
+        :return: Available contexts list
+        :rtype: List[Context]
+        """
+        contexts_map = cls.contexts_map()
+        if contexts_map is None:
+            return []
+        return list(contexts_map.values())
+
+    @classmethod
+    def get_context(cls, context_identifier: str) -> Context:
         """
         Get configuration dictionary of a context.
 
-        :param context_name: Name of the desired context.
-        :type context_name: str
+        :param context_identifier: Name or alias of the desired context.
+        :type context_identifier: str
         :return: configuration dictionary.
-        :raises: raise :Class:`MissingConfiguration` if no contexts configuration was
-        set.
+        :rtype: Context
+        :raises MissingConfiguration: raised if no contexts configuration was set.
+        :raises UnknownContext: raised if context was not found.
         """
-        contexts_configuration = cls.contexts_configuration()
+        contexts_configuration = cls.contexts_map()
         if contexts_configuration is None:
             raise MissingConfiguration(CONTEXTS)
-        return contexts_configuration.get(context_name, None)
+        for context_name, context in contexts_configuration.items():
+            if (
+                context_identifier == context_name
+                or context_identifier in context.aliases  # noqa: disable=W503
+            ):
+                return context
+        raise UnknownContext(context_identifier)
 
     @classmethod
     def read_commands(
@@ -171,12 +250,16 @@ class Configuration:
         deny_list: Optional[List[str]] = None,
     ) -> List[Command]:
         """
-        Read commands from a settings file.
+        Read commands with given constraints.
 
-        :param contexts: List of str. a list of contexts to choose commands from.
+        :param contexts: List of contexts to choose commands from.
+        :type contexts: None or List[str]
         :param allow_list: List of allowed commands. If None, take all commands
+        :type allow_list: None or List[str]
         :param deny_list: List of denied commands. If None, take all commands
-        :return: a list of :class:`Command`
+        :type deny_list: None or List[str]
+        :return: List of commands according to constraints
+        :rtype: List[Command]
         """
         commands = []
         contexts = [] if contexts is None else contexts
@@ -203,33 +286,49 @@ class Configuration:
         deny_list: Optional[List[str]] = None,
     ) -> Command:
         """
-        Read command from a settings file.
+        Read command with given constraints.
 
-        :param command_name: the name of the command to read.
-        :param contexts: List of str. a list of contexts to choose commands from.
+        :param command_name: Name of the command to read
+        :type command_name: str
+        :param contexts: List of contexts to choose commands from.
+        :type contexts: None or List[str]
         :param allow_list: List of allowed commands. If None, take all commands
+        :type allow_list: None or List[str]
         :param deny_list: List of denied commands. If None, take all commands
-        :return: a :class:`Command` instance
-        :raises: :class:`UnknownCommand` if command is missing from settings file.
-        :class:`InvalidCommand` of command doesn't fit the given contexts, allow list
-         and deny list
+        :type deny_list: None or List[str]
+        :return: Command instance with constraints
+        :rtype: Command
+        :raises UnknownCommand: raised if command is missing from settings file.
+        :raises InvalidCommand: raised if command doesn't fit the given contexts,
+            allow list or deny list
         """
+        if (
+            allow_list is not None
+            and len(allow_list) != 0  # noqa: W503
+            and command_name not in allow_list  # noqa: W503
+        ):
+            raise InvalidCommand(
+                f'Command "{command_name}" '
+                f"was not specified in allowed list: {', '.join(allow_list)}"
+            )
+        if deny_list is not None and command_name in deny_list:
+            raise InvalidCommand(
+                f'Command "{command_name}" '
+                f"was explicitly denied in deny list: {', '.join(deny_list)}"
+            )
         command_configuration = cls.get_command_configuration(command_name)
         if command_configuration is None:
             raise UnknownCommand(command_name)
-        if not cls.__is_command_matching(
-            command_name, command_configuration, contexts, allow_list, deny_list
-        ):
-            raise InvalidCommand(
-                command_name=command_name,
-                contexts=contexts,
-                allow_list=allow_list,
-                deny_list=deny_list,
-            )
-        contexts = [] if contexts is None else contexts
-        for context in contexts:
-            context_obj = command_configuration.get(context, None)
-            if not isinstance(context_obj, dict):
+        if contexts is None or len(contexts) == 0:
+            contexts = [STANDARD]
+        context_objects = [cls.get_context(context_name) for context_name in contexts]
+        for context in context_objects:
+            context_obj = context.search_context(command_configuration)
+            if context_obj is False or context_obj is None:
+                raise InvalidCommand(
+                    f'Command "{command_name}" does not match context "{context.name}"'
+                )
+            if context_obj is True:
                 continue
             command_configuration = cls.__combine_command_setups(
                 command_configuration, context_obj
@@ -238,6 +337,7 @@ class Configuration:
             name=command_name,
             args=command_configuration.get(ARGS, []),
             help=command_configuration[HELP],
+            version=command_configuration.get(VERSION, None),
         )
 
     @classmethod
@@ -252,10 +352,12 @@ class Configuration:
         from configuration file.
 
         :param statue_configuration_path: User-defined file path containing
-        repository-specific configurations
+            repository-specific configurations
+        :type statue_configuration_path: None, str or Path
         """
         if statue_configuration_path is None:
-            statue_configuration_path = cls.configuration_path(Path.cwd())
+            cwd = Path.cwd()
+            statue_configuration_path = cls.configuration_path(cwd)
         if isinstance(statue_configuration_path, str):
             statue_configuration_path = Path(statue_configuration_path)
         cls.set_statue_configuration(
@@ -272,7 +374,12 @@ class Configuration:
     def __load_default_configuration(cls) -> None:
         if not DEFAULT_CONFIGURATION_FILE.exists():
             return
-        cls.set_default_configuration(toml.load(DEFAULT_CONFIGURATION_FILE))
+        default_configuration = toml.load(DEFAULT_CONFIGURATION_FILE)
+        if CONTEXTS in default_configuration:
+            default_configuration[CONTEXTS] = Context.build_contexts_map(
+                default_configuration[CONTEXTS]
+            )
+        cls.set_default_configuration(default_configuration)
 
     @classmethod
     def __build_configuration(
@@ -286,7 +393,10 @@ class Configuration:
         from configuration file.
 
         :param statue_configuration_path: User-defined file path containing
-        repository-specific configurations
+            repository-specific configurations
+        :type statue_configuration_path: Path
+        :return: Configuration mutable map
+        :rtype: None or MutableMapping[str, Any]
         """
         if not statue_configuration_path.exists():
             return None
@@ -303,12 +413,20 @@ class Configuration:
         )
         if commands_configuration is not None:
             statue_config[COMMANDS] = commands_configuration
-        contexts_configuration = cls.__build_contexts_configuration(
-            statue_contexts_configuration=statue_config.get(CONTEXTS, None),
-            default_contexts_configuration=default_configuration.get(CONTEXTS, None),
-        )
-        if contexts_configuration is not None:
-            statue_config[CONTEXTS] = contexts_configuration
+        contexts_configuration = statue_config.get(CONTEXTS, None)
+        if contexts_configuration is None:
+            contexts_map = default_configuration.get(CONTEXTS, None)
+        else:
+            contexts_map = Context.build_contexts_map(
+                contexts_configuration,
+                base_contexts_map=default_configuration.get(CONTEXTS, None),
+            )
+        if contexts_map is not None:
+            statue_config[CONTEXTS] = contexts_map
+        if SOURCES in statue_config:
+            statue_config[SOURCES] = {
+                Path(source): setup for source, setup in statue_config[SOURCES].items()
+            }
         return statue_config
 
     @classmethod
@@ -331,79 +449,6 @@ class Configuration:
                 commands_configuration[command_name] = command_setups
         return commands_configuration
 
-    @classmethod
-    def __build_contexts_configuration(
-        cls,
-        statue_contexts_configuration: Optional[MutableMapping[str, Any]],
-        default_contexts_configuration: Optional[MutableMapping[str, Any]],
-    ) -> Optional[MutableMapping[str, Any]]:
-        if statue_contexts_configuration is None:
-            return default_contexts_configuration
-        if default_contexts_configuration is None:
-            return statue_contexts_configuration
-        contexts = deepcopy(default_contexts_configuration)
-        for context_name, context_setup in statue_contexts_configuration.items():
-            if context_name in contexts:
-                raise InvalidStatueConfiguration(
-                    f'"{context_name}" is a predefined context and cannot be override'
-                )
-            contexts[context_name] = context_setup
-        return contexts
-
-    @classmethod
-    def __is_command_matching(  # pylint: disable=too-many-arguments
-        cls,
-        command_name: str,
-        setups: MutableMapping[str, Any],
-        contexts: Optional[List[str]],
-        allow_list: Optional[List[str]],
-        deny_list: Optional[List[str]],
-    ) -> bool:
-        """
-        Check whether a command fits the restrictions or not.
-
-        :param command_name: the name of the command to read.
-        :param setups: Dictionary. The command's configuration.
-        :param contexts: List of str. a list of contexts.
-        :param allow_list: List of allowed commands.
-        :param deny_list: List of denied commands.
-        :return: Boolean. Does the command fit the restrictions
-        """
-        if deny_list is not None and command_name in deny_list:
-            return False
-        if (
-            allow_list is not None
-            and len(allow_list) != 0  # noqa: W503
-            and command_name not in allow_list  # noqa: W503
-        ):
-            return False
-        if contexts is None or len(contexts) == 0:
-            return cls.__command_match_default_context(setups)
-        for command_context in contexts:
-            if not cls.__command_match_context(setups, command_context):
-                return False
-        return True
-
-    @classmethod
-    def __command_match_context(
-        cls, setups: MutableMapping[str, Any], context_name: str
-    ) -> bool:
-        if context_name == STANDARD:
-            return cls.__command_match_default_context(setups)
-        context_configuration = cls.get_context_configuration(context_name)
-        if context_configuration is None:
-            raise UnknownContext(context_name)
-        if setups.get(context_name, False):
-            return True
-        parent_context = context_configuration.get(PARENT, None)
-        if parent_context is not None:
-            return cls.__command_match_context(setups, parent_context)
-        return False
-
-    @classmethod
-    def __command_match_default_context(cls, setups: MutableMapping[str, Any]) -> bool:
-        return setups.get(STANDARD, True)
-
     @classmethod
     def __combine_command_setups(
         cls,
